/*
正体補正プログラム
					2014/7/10 by S.Ushimaru
超音波センサの番号は、次のようになっていることを前提としている。
	USS1:正面左側     USS2:正面右側

*/

#include <linux/errno.h>
#include <sys/io.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include <pthread.h>
#include <stdio.h>
#include "hardware.h"
#include "uss.h"
#include "run_ctrl.h"
#include "direction.h"

static float uss_dist = 17.0 ; //12.5; //超音波センサ間の距離[cm]

static float ref_angle;

/* 引数 dist：正体補正する物体までの距離 */
int direction_correct(int dist){

	int uss_l; //left
	int uss_r; //right
	int diff_uss;
	float diff_max=10;
	int c_count=0;
	int c_count_max=10; // 正体補正を諦めるまでの回数
	int cur_dist;
	int ref_dist = 0; 
	//所定距離：距離補正に入る際の超音波センサの測定範囲
	int uss_max = 80; //所定距離の最大値[cm]
	int uss_min = 30; //所定距離の最小値[cm]

	float str_speed = 10.0; //距離補正時の速度[cm/s]
	float rot_speed = 5.0; //正体補正時の回転速度[rad/s]
	float angle_coef = 0.8; //正体補正時の回転角度調整係数


	float resol;

	printf("Direction Correction Start !!1\n") ;

	resol = 1. / uss_dist * 180.0 /3.1415; // [deg]
	//printf("angle resoution =%8.3f\n",resol) ;

// 距離の補正

	if(dist!=0){

		printf("距離補正スタート\n");

		ref_dist = dist;
	
		uss_l=uss_get(1); // left
		usleep(50000);
		uss_r=uss_get(2); // right
		usleep(50000);

		//両方のUSSの距離を信頼する場合
		if((uss_l > uss_min  || uss_l < uss_max ) && (uss_r > uss_min  || uss_r < uss_max)){
			cur_dist =(uss_l + uss_r)/2;
		} // 左のUSSの距離を信頼する場合
		else if(uss_l > uss_min  || uss_l < uss_max){
			cur_dist = uss_l;
		} // 右のUSSの距離を信頼する場合
		else if(uss_r > uss_min  || uss_r < uss_max){
			cur_dist = uss_r;
		}
		else{ //両方とも信頼出来ない場合
			cur_dist = 999;
		}

		printf("ref_dist=%d     cur_dist = %d\n",ref_dist,cur_dist);

		if(cur_dist!=999){
			run_ctrl_create();
			if(cur_dist - ref_dist > 0){
				run_ctrl_set(STRAIGHT, str_speed);
			}else{
				run_ctrl_set(STRAIGHT, -str_speed);
			}
                	run_ctrl_set_dist((float)(cur_dist - ref_dist));
			while(1){
				//printf("run straight loop \n");
				if(run_ctrl_get_endflag()==1){
					//run_ctrl_reset();
					break;
				}
				usleep(20000);
			}
			run_ctrl_cancel();

		}else{
			printf("can't get distance using USS\n");
			return -1;
		}


		usleep(1000*1000); //1秒待つ

	}


// 正体補正
	
	run_ctrl_reset();
	printf("正体補正スタート\n");

	while(1){
	
		//printf("-----------------------------------------------\n");
		//printf("correction count =%d\n",c_count);
		uss_l=uss_get(1); // left
		usleep(50000);
		uss_r=uss_get(2); // right
		usleep(50000);
		printf("uss_l=%d   uss_r= %d\n",uss_l, uss_r) ;

		//一時的に回転減速域のゲインを変更する
		//run_ctrl_set_Kfr(1.0);

		diff_uss = uss_l - uss_r;

		//正体補正したと判断する条件 : USSの差を 0 とするか abs(1) とするか
		//if(diff_uss == 0) break;
		if( (diff_uss >= -1) && (diff_uss <=1) ) break;

		if(diff_uss > diff_max) diff_uss = diff_max;
		else if (diff_uss < -diff_max) diff_uss = -diff_max;

		// 超音波センサ計測距離の差から回転角を計算、ただし angle_coef で調整
		ref_angle = diff_uss / uss_dist * 180.0 /3.1415 * angle_coef ; // [deg]

		printf("diff_uss=%d      ref_angle=%6.1f\n",diff_uss, ref_angle);

		run_ctrl_create();
		if(ref_angle > 0){
			run_ctrl_set(ROTATE, rot_speed);
		}else{
			run_ctrl_set(ROTATE, -rot_speed);
		}
                run_ctrl_set_dist(ref_angle);

		while(1){
			//printf("run roate loop \n");
			if(run_ctrl_get_endflag()==1){
				//run_ctrl_reset();
				break;
			}
			usleep(20000);
		}

		c_count++;

		if(c_count > c_count_max) break;

		usleep(100*1000); // 100ms待つ

	}
	run_ctrl_stop();
	run_ctrl_reset_Kfr();

	if(c_count <= c_count_max){
		return 1;
	}else{
		return -1;
	}

	//return 0;

}

